package fr.soleil.docking.vl;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import javax.swing.JComponent;
import javax.swing.JOptionPane;

import com.vldocking.swing.docking.Dockable;
import com.vldocking.swing.docking.DockingDesktop;

import fr.soleil.docking.ADockingManager;
import fr.soleil.docking.perspective.IPerspective;
import fr.soleil.docking.perspective.PerspectiveFactory;
import fr.soleil.docking.view.ViewFactory;

public class VlDockDockingManager extends ADockingManager {
    protected DockingDesktop rootWindow;

    public VlDockDockingManager() {
        super(new ViewFactory(), new PerspectiveFactory());
//	        ((VlDockViewFactory) this.viewFactory).setManager(this);
    }

    @Override
    public JComponent getDockingArea() {
        if (rootWindow == null) {
            rootWindow = (DockingDesktop) createNewDockingArea(null);
            rootWindow.setOpaqueContents(true);
        }
        return rootWindow;
    }

    public DockingDesktop getDockingDesktop() {
        return (DockingDesktop) getDockingArea();
    }

    @Override
    public JComponent createNewDockingArea(Color background) {
        DockingDesktop result = new DockingDesktop();
        if (background != null) {
            result.setBackground(background);
        }
        return result;
    }

    // /**
    // * Creates a new docking area with a given background {@link Color}
    // *
    // * @param background The background {@link Color}
    // * @return A {@link JComponent}
    // */
    // @Override
    // public JComponent createNewDockingArea(Color background) {
    // DockingDesktop result = (DockingDesktop) createNewDockingArea();
    // if (background != null) {
    // result.setBackground(background);
    // }
    // return result;
    // }

    @Override
    public void loadPerspective(IPerspective perspective) {
        if (perspective.getByteArray().length > 0) {

            ObjectInputStream ois = null;
            ByteArrayInputStream bais = null;
            try {

                bais = new ByteArrayInputStream(perspective.getByteArray());
                ois = new ObjectInputStream(bais);
                rootWindow.readXML(ois);
            } catch (Exception e) {
                JOptionPane.showMessageDialog(rootWindow, "Perspective.load() :" + " Unexpected Error (see traces)",
                        "INFONODE Docking - Error", JOptionPane.ERROR_MESSAGE);
            } finally {
                try {
                    if (ois != null) {
                        ois.close();
                    }
                    // Actually there is no effect to close bais ...
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    @Override
    protected void savePerspective(IPerspective perspective) {
        ByteArrayOutputStream baos = null;
        ObjectOutputStream ous = null;
        baos = new ByteArrayOutputStream();
        try {
            ous = new ObjectOutputStream(baos);
            rootWindow.writeXML(ous);
        } catch (IOException e) {
            e.printStackTrace();
            JOptionPane.showMessageDialog(rootWindow, "Perspective.save() :" + " Unexpected Error (see traces)",
                    "INFONODE Docking - Error", JOptionPane.ERROR_MESSAGE);
        } finally {
            try {
                ous.close();
                // Actually there is no effect to close baos ...
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        perspective.setByteArray(baos.toByteArray());

    }

    @Override
    protected void updateViews(PropertyChangeEvent evt) {
        if (evt.getSource() == this.viewFactory) {
            Dockable newView = (Dockable) evt.getNewValue();
            Dockable oldView = (Dockable) evt.getOldValue();
            if (newView == null) { // removed
                rootWindow.remove(oldView);
            } else {
                rootWindow.addDockable(newView);
            }
        }
    }

    @Override
    public void setEnabledDocking(boolean enabledDocking) {
        //
    }

    @Override
    public boolean isEnabledDocking() {
        return true;
    }

}
